@@ -41,6 +41,7 @@ def register():
         JournalView,
         JournalViewColumn,
         Journal,
+        OpenJournalCashStart,
         JournalPeriod,
         Move,
         Reconciliation,
@@ -90,6 +91,7 @@ def register():
         OpenAgedBalance,
         ClosePeriod,
         ReOpenPeriod,
+        OpenJournalCash,
         CloseJournalPeriod,
         ReOpenJournalPeriod,
         OpenJournal,
