@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from dateutil.relativedelta import relativedelta
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, StateTransition, StateAction, \
@@ -231,12 +231,11 @@ class FiscalYear(ModelSQL, ModelView):
                 return None
         return fiscalyears[0].id
 
-    def _process_account(self, account):
-        '''
-        Process account for a fiscal year closed
-        '''
-        Currency = Pool().get('currency.currency')
-        Deferral = Pool().get('account.account.deferral')
+    def get_deferral(self, account):
+        'Computes deferrals for accounts'
+        pool = Pool()
+        Currency = pool.get('currency.currency')
+        Deferral = pool.get('account.account.deferral')
 
         if account.kind == 'view':
             return
@@ -245,12 +244,12 @@ class FiscalYear(ModelSQL, ModelView):
                 self.raise_user_error('account_balance_not_zero',
                         error_args=(account.rec_name,))
         else:
-            Deferral.create([{
-                        'account': account.id,
-                        'fiscalyear': self.id,
-                        'debit': account.debit,
-                        'credit': account.credit,
-                        }])
+            deferral = Deferral()
+            deferral.account = account
+            deferral.fiscalyear = self
+            deferral.debit = account.debit
+            deferral.credit = account.credit
+            return deferral
 
     @classmethod
     @ModelView.button
@@ -261,7 +260,9 @@ class FiscalYear(ModelSQL, ModelView):
         pool = Pool()
         Period = pool.get('account.period')
         Account = pool.get('account.account')
+        Deferral = pool.get('account.account.deferral')
 
+        deferrals = []
         for fiscalyear in fiscalyears:
             if cls.search([
                         ('end_date', '<=', fiscalyear.start_date),
@@ -270,8 +271,8 @@ class FiscalYear(ModelSQL, ModelView):
                         ]):
                 cls.raise_user_error('close_error', (fiscalyear.rec_name,))
 
-            #First close the fiscalyear to be sure
-            #it will not have new period created between.
+            # First close the fiscalyear to be sure
+            # it will not have new period created between.
             cls.write([fiscalyear], {
                 'state': 'close',
                 })
@@ -285,8 +286,9 @@ class FiscalYear(ModelSQL, ModelView):
                 accounts = Account.search([
                         ('company', '=', fiscalyear.company.id),
                         ])
-            for account in accounts:
-                fiscalyear._process_account(account)
+                deferrals += filter(None, (fiscalyear.get_deferral(a)
+                        for a in accounts))
+        Deferral.save(deferrals)
 
     @classmethod
     @ModelView.button
@@ -315,7 +317,11 @@ class FiscalYear(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('code',) + tuple(clause[1:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
@@ -326,6 +332,8 @@ class BalanceNonDeferralStart(ModelView):
     __name__ = 'account.fiscalyear.balance_non_deferral.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
         required=True, domain=[('state', '=', 'open')])
+    company = fields.Function(fields.Many2One('company.company', 'Company'),
+        'on_change_with_company')
     journal = fields.Many2One('account.journal', 'Journal', required=True,
         domain=[
             ('type', '=', 'situation'),
@@ -340,16 +348,23 @@ class BalanceNonDeferralStart(ModelView):
         required=True,
         domain=[
             ('kind', '!=', 'view'),
-            ('company', '=', Eval('context', {}).get('company', -1)),
+            ('company', '=', Eval('company', -1)),
             ('deferral', '=', True),
-            ])
+            ],
+        depends=['company'])
     debit_account = fields.Many2One('account.account', 'Debit Account',
         required=True,
         domain=[
             ('kind', '!=', 'view'),
-            ('company', '=', Eval('context', {}).get('company', -1)),
+            ('company', '=', Eval('company', -1)),
             ('deferral', '=', True),
-            ])
+            ],
+        depends=['company'])
+
+    @fields.depends('fiscalyear')
+    def on_change_with_company(self, name=None):
+        if self.fiscalyear:
+            return self.fiscalyear.company.id
 
 
 class BalanceNonDeferral(Wizard):
@@ -358,23 +373,25 @@ class BalanceNonDeferral(Wizard):
     start = StateView('account.fiscalyear.balance_non_deferral.start',
         'account.fiscalyear_balance_non_deferral_start_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Ok', 'balance', 'tryton-ok', default=True),
+            Button('OK', 'balance', 'tryton-ok', default=True),
             ])
     balance = StateAction('account.act_move_line_form')
 
     def get_move_line(self, account):
         pool = Pool()
         Line = pool.get('account.move.line')
-        if account.company.currency.is_zero(account.balance):
+        # Don't use account.balance because we need the non-commulated balance
+        balance = account.debit - account.credit
+        if account.company.currency.is_zero(balance):
             return
         line = Line()
         line.account = account
-        if account.balance >= 0:
-            line.credit = abs(account.balance)
+        if balance >= 0:
+            line.credit = abs(balance)
             line.debit = 0
         else:
             line.credit = 0
-            line.debit = abs(account.balance)
+            line.debit = abs(balance)
         return line
 
     def get_counterpart_line(self, amount):
