@@ -10,12 +10,12 @@ from sql import Null
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce, Case
 
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import ModelView, ModelSQL, fields, Check
 from trytond.wizard import Wizard, StateTransition, StateView, StateAction, \
-    Button
+    StateReport, Button
 from trytond.report import Report
 from trytond import backend
-from trytond.pyson import Eval, Bool, PYSONEncoder, If
+from trytond.pyson import Eval, Bool, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.rpc import RPC
@@ -96,7 +96,7 @@ class Move(ModelSQL, ModelView):
                 'modify_posted_move': ('You can not modify move "%s" because '
                     'it is already posted.'),
                 'date_outside_period': ('You can not create move "%(move)s" '
-                    'because it\'s date is outside its period.'),
+                    'because its date is outside its period.'),
                 'draft_closed_period': ('You can not set to draft move '
                     '"%(move)s" because period "%(period)s" is closed.'),
                 'period_cancel': (
@@ -469,14 +469,14 @@ class Reconciliation(ModelSQL, ModelView):
                 'reconciliation_line_not_valid': ('You can not reconcile line '
                     '"%s" because it is not in valid state.'),
                 'reconciliation_different_accounts': ('You can not reconcile '
-                    'line "%(line)s" because it\'s account "%(account1)s" is '
+                    'line "%(line)s" because its account "%(account1)s" is '
                     'different from "%(account2)s".'),
                 'reconciliation_account_no_reconcile': (
                     'You can not reconcile '
-                    'line "%(line)s" because it\'s account "%(account)s" is '
+                    'line "%(line)s" because its account "%(account)s" is '
                     'configured as not reconcilable.'),
                 'reconciliation_different_parties': ('You can not reconcile '
-                    'line "%(line)s" because it\'s party "%(party1)s" is '
+                    'line "%(line)s" because its party "%(party1)s" is '
                     'different from "%(party2)s".'),
                 'reconciliation_unbalanced': ('You can not create a '
                     'reconciliation where debit "%(debit)s" and credit '
@@ -643,13 +643,14 @@ class Line(ModelSQL, ModelView):
         cls._reconciliation_modify_disallow = {
             'account', 'debit', 'credit', 'party',
             }
+        table = cls.__table__()
         cls._sql_constraints += [
             ('credit_debit',
-                'CHECK(credit * debit = 0.0)',
+                Check(table, table.credit * table.debit == 0),
                 'Wrong credit/debit values.'),
             ('second_currency_sign',
-                'CHECK(COALESCE(amount_second_currency, 0) '
-                '* (debit - credit) >= 0)',
+                Check(table, Coalesce(table.amount_second_currency, 0)
+                    * (table.debit - table.credit) >= 0),
                 'wrong_second_currency_sign'),
             ]
         cls.__rpc__.update({
@@ -1481,8 +1482,8 @@ class Line(ModelSQL, ModelView):
                 return result
 
             xml = '<?xml version="1.0"?>\n' \
-                '<tree string="%s" editable="top" on_write="on_write" ' \
-                'colors="red:state==\'draft\'">\n' % title
+                '<tree string="%s" editable="top" on_write="on_write">\n' \
+                % title
             fields = set()
             for column in journal.view.columns:
                 fields.add(column.field.name)
@@ -1507,11 +1508,6 @@ class Line(ModelSQL, ModelView):
             result['fields'] = cls.fields_get(fields_names=list(fields))
         return result
 
-    @classmethod
-    def view_attributes(cls):
-        return [('/tree[@on_write="on_write"]', 'colors',
-                If(Eval('state') == 'draft', 'red', 'black'))]
-
     @classmethod
     def reconcile(cls, lines, journal=None, date=None, account=None,
             description=None):
@@ -2079,7 +2075,7 @@ class PrintGeneralJournal(Wizard):
             Button('Cancel', 'end', 'tryton-cancel'),
             Button('Print', 'print_', 'tryton-print', default=True),
             ])
-    print_ = StateAction('account.report_general_journal')
+    print_ = StateReport('account.move.general_journal')
 
     def do_print_(self, action):
         data = {
